@@ -184,9 +184,9 @@ class RandomState(object):
                           size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.gamma)
-    def gamma(self, shape, scale=1.0, chunks=None):
-        return self._wrap(np.random.RandomState.gamma, scale,
-                          size=shape, chunks=chunks)
+    def gamma(self, shape, scale=1.0, size=None, chunks=None):
+        return self._wrap(np.random.RandomState.gamma, shape, scale,
+                          size=size, chunks=chunks)
 
     @doc_wraps(np.random.RandomState.geometric)
     def geometric(self, p, size=None, chunks=None):
