@@ -28,16 +28,19 @@ def reduction(x, chunk, aggregate, axis=None, keepdims=None, dtype=None,
         axis = tuple(range(x.ndim))
     if isinstance(axis, int):
         axis = (axis,)
-    axis = tuple(i if i >= 0 else x.ndim + i for i in axis)
+    axis = tuple(validate_axis(x.ndim, a) for a in axis)
 
-    if dtype is not None and 'dtype' in getargspec(chunk).args:
+    if dtype is None:
+        raise ValueError("Must specify dtype")
+    if 'dtype' in getargspec(chunk).args:
         chunk = partial(chunk, dtype=dtype)
-    if dtype is not None and 'dtype' in getargspec(aggregate).args:
+    if 'dtype' in getargspec(aggregate).args:
         aggregate = partial(aggregate, dtype=dtype)
 
     # Map chunk across all blocks
     inds = tuple(range(x.ndim))
-    tmp = atop(chunk, inds, x, inds, axis=axis, keepdims=True)
+    # The dtype of `tmp` doesn't actually matter, and may be incorrect.
+    tmp = atop(chunk, inds, x, inds, axis=axis, keepdims=True, dtype=x.dtype)
     tmp._chunks = tuple((1, ) * len(c) if i in axis else c for (i, c)
                         in enumerate(tmp.chunks))
 
@@ -69,12 +72,11 @@ def _tree_reduce(x, aggregate, axis, keepdims, dtype, split_every=None,
     func = compose(partial(combine or aggregate, axis=axis, keepdims=True),
                    partial(_concatenate2, axes=axis))
     for i in range(depth - 1):
-        x = partial_reduce(func, x, split_every, True, None,
+        x = partial_reduce(func, x, split_every, True, dtype=dtype,
                            name=(name or funcname(combine or aggregate)) + '-partial')
     func = compose(partial(aggregate, axis=axis, keepdims=keepdims),
                    partial(_concatenate2, axes=axis))
-    return partial_reduce(func, x, split_every, keepdims=keepdims,
-                          dtype=dtype,
+    return partial_reduce(func, x, split_every, keepdims=keepdims, dtype=dtype,
                           name=(name or funcname(aggregate)) + '-aggregate')
 
 
@@ -120,10 +122,8 @@ def partial_reduce(func, x, split_every, keepdims=False, dtype=None, name=None):
 def sum(a, axis=None, dtype=None, keepdims=False, split_every=None):
     if dtype is not None:
         dt = dtype
-    elif a._dtype is not None:
-        dt = np.empty((1,), dtype=a._dtype).sum().dtype
     else:
-        dt = None
+        dt = np.empty((1,), dtype=a.dtype).sum().dtype
     return reduction(a, chunk.sum, chunk.sum, axis=axis, keepdims=keepdims,
                      dtype=dt, split_every=split_every)
 
@@ -132,10 +132,8 @@ def sum(a, axis=None, dtype=None, keepdims=False, split_every=None):
 def prod(a, axis=None, dtype=None, keepdims=False, split_every=None):
     if dtype is not None:
         dt = dtype
-    elif a._dtype is not None:
-        dt = np.empty((1,), dtype=a._dtype).prod().dtype
     else:
-        dt = None
+        dt = np.empty((1,), dtype=a.dtype).prod().dtype
     return reduction(a, chunk.prod, chunk.prod, axis=axis, keepdims=keepdims,
                      dtype=dt, split_every=split_every)
 
@@ -143,13 +141,13 @@ def prod(a, axis=None, dtype=None, keepdims=False, split_every=None):
 @wraps(chunk.min)
 def min(a, axis=None, keepdims=False, split_every=None):
     return reduction(a, chunk.min, chunk.min, axis=axis, keepdims=keepdims,
-                     dtype=a._dtype, split_every=split_every)
+                     dtype=a.dtype, split_every=split_every)
 
 
 @wraps(chunk.max)
 def max(a, axis=None, keepdims=False, split_every=None):
     return reduction(a, chunk.max, chunk.max, axis=axis, keepdims=keepdims,
-                     dtype=a._dtype, split_every=split_every)
+                     dtype=a.dtype, split_every=split_every)
 
 
 @wraps(chunk.any)
@@ -168,10 +166,8 @@ def all(a, axis=None, keepdims=False, split_every=None):
 def nansum(a, axis=None, dtype=None, keepdims=False, split_every=None):
     if dtype is not None:
         dt = dtype
-    elif a._dtype is not None:
-        dt = chunk.nansum(np.empty((1,), dtype=a._dtype)).dtype
     else:
-        dt = None
+        dt = chunk.nansum(np.empty((1,), dtype=a.dtype)).dtype
     return reduction(a, chunk.nansum, chunk.sum, axis=axis, keepdims=keepdims,
                      dtype=dt, split_every=split_every)
 
@@ -181,10 +177,8 @@ with ignoring(AttributeError):
     def nanprod(a, axis=None, dtype=None, keepdims=False, split_every=None):
         if dtype is not None:
             dt = dtype
-        elif a._dtype is not None:
-            dt = chunk.nanprod(np.empty((1,), dtype=a._dtype)).dtype
         else:
-            dt = None
+            dt = chunk.nanprod(np.empty((1,), dtype=a.dtype)).dtype
         return reduction(a, chunk.nanprod, chunk.prod, axis=axis,
                          keepdims=keepdims, dtype=dt, split_every=split_every)
 
@@ -200,13 +194,13 @@ with ignoring(AttributeError):
 @wraps(chunk.nanmin)
 def nanmin(a, axis=None, keepdims=False, split_every=None):
     return reduction(a, chunk.nanmin, chunk.nanmin, axis=axis,
-                     keepdims=keepdims, dtype=a._dtype, split_every=split_every)
+                     keepdims=keepdims, dtype=a.dtype, split_every=split_every)
 
 
 @wraps(chunk.nanmax)
 def nanmax(a, axis=None, keepdims=False, split_every=None):
     return reduction(a, chunk.nanmax, chunk.nanmax, axis=axis,
-                     keepdims=keepdims, dtype=a._dtype, split_every=split_every)
+                     keepdims=keepdims, dtype=a.dtype, split_every=split_every)
 
 
 def numel(x, **kwargs):
@@ -247,10 +241,8 @@ def mean_agg(pair, dtype='f8', **kwargs):
 def mean(a, axis=None, dtype=None, keepdims=False, split_every=None):
     if dtype is not None:
         dt = dtype
-    elif a._dtype is not None:
-        dt = np.mean(np.empty(shape=(1,), dtype=a._dtype)).dtype
     else:
-        dt = None
+        dt = np.mean(np.empty(shape=(1,), dtype=a.dtype)).dtype
     return reduction(a, mean_chunk, mean_agg, axis=axis, keepdims=keepdims,
                      dtype=dt, split_every=split_every, combine=mean_combine)
 
@@ -258,15 +250,14 @@ def mean(a, axis=None, dtype=None, keepdims=False, split_every=None):
 def nanmean(a, axis=None, dtype=None, keepdims=False, split_every=None):
     if dtype is not None:
         dt = dtype
-    elif a._dtype is not None:
-        dt = np.mean(np.empty(shape=(1,), dtype=a._dtype)).dtype
     else:
-        dt = None
+        dt = np.mean(np.empty(shape=(1,), dtype=a.dtype)).dtype
     return reduction(a, partial(mean_chunk, sum=chunk.nansum, numel=nannumel),
                      mean_agg, axis=axis, keepdims=keepdims, dtype=dt,
                      split_every=split_every,
                      combine=partial(mean_combine, sum=chunk.nansum, numel=nannumel))
 
+
 with ignoring(AttributeError):
     nanmean = wraps(chunk.nanmean)(nanmean)
 
@@ -345,10 +336,8 @@ def moment(a, order, axis=None, dtype=None, keepdims=False, ddof=0,
         raise ValueError("Order must be an integer >= 2")
     if dtype is not None:
         dt = dtype
-    elif a._dtype is not None:
-        dt = np.var(np.ones(shape=(1,), dtype=a._dtype)).dtype
     else:
-        dt = None
+        dt = np.var(np.ones(shape=(1,), dtype=a.dtype)).dtype
     return reduction(a, partial(moment_chunk, order=order),
                      partial(moment_agg, order=order, ddof=ddof),
                      axis=axis, keepdims=keepdims,
@@ -360,10 +349,8 @@ def moment(a, order, axis=None, dtype=None, keepdims=False, ddof=0,
 def var(a, axis=None, dtype=None, keepdims=False, ddof=0, split_every=None):
     if dtype is not None:
         dt = dtype
-    elif a._dtype is not None:
-        dt = np.var(np.ones(shape=(1,), dtype=a._dtype)).dtype
     else:
-        dt = None
+        dt = np.var(np.ones(shape=(1,), dtype=a.dtype)).dtype
     return reduction(a, moment_chunk, partial(moment_agg, ddof=ddof), axis=axis,
                      keepdims=keepdims, dtype=dt, split_every=split_every,
                      combine=moment_combine, name='var')
@@ -372,10 +359,8 @@ def var(a, axis=None, dtype=None, keepdims=False, ddof=0, split_every=None):
 def nanvar(a, axis=None, dtype=None, keepdims=False, ddof=0, split_every=None):
     if dtype is not None:
         dt = dtype
-    elif a._dtype is not None:
-        dt = np.var(np.ones(shape=(1,), dtype=a._dtype)).dtype
     else:
-        dt = None
+        dt = np.var(np.ones(shape=(1,), dtype=a.dtype)).dtype
     return reduction(a, partial(moment_chunk, sum=chunk.nansum, numel=nannumel),
                      partial(moment_agg, sum=np.nansum, ddof=ddof), axis=axis,
                      keepdims=keepdims, dtype=dt, split_every=split_every,
@@ -402,6 +387,7 @@ def nanstd(a, axis=None, dtype=None, keepdims=False, ddof=0, split_every=None):
         result = result.astype(dtype)
     return result
 
+
 with ignoring(AttributeError):
     nanstd = wraps(chunk.nanstd)(nanstd)
 
@@ -532,7 +518,8 @@ def arg_reduction(x, chunk, combine, agg, axis=None, split_every=None):
                    in enumerate(x.chunks))
     dsk = dict(((name,) + k, (chunk, (old,) + k, axis, off)) for (k, off)
                in zip(keys, offset_info))
-    tmp = Array(merge(dsk, x.dask), name, chunks)
+    # The dtype of `tmp` doesn't actually matter, just need to provide something
+    tmp = Array(merge(dsk, x.dask), name, chunks, dtype=x.dtype)
     return _tree_reduce(tmp, agg, axis, False, np.int64, split_every, combine)
 
 
@@ -607,6 +594,8 @@ def cumreduction(func, binop, ident, x, axis, dtype=None):
     if dtype is None:
         dtype = func(np.empty((0,), dtype=x.dtype)).dtype
     assert isinstance(axis, int)
+    axis = validate_axis(x.ndim, axis)
+
     m = x.map_blocks(func, axis=axis, dtype=dtype)
 
     name = '%s-axis=%d-%s' % (func.__name__, axis, tokenize(x, dtype))
@@ -644,3 +633,14 @@ def cumsum(x, axis, dtype=None):
 @wraps(np.cumprod)
 def cumprod(x, axis, dtype=None):
     return cumreduction(np.cumprod, operator.mul, 1, x, axis, dtype)
+
+
+def validate_axis(ndim, axis):
+    """ Validate single axis dimension against number of dimensions """
+    if axis > ndim - 1 or axis < -ndim:
+        raise ValueError("Axis must be between -%d and %d, got %d" %
+                         (ndim, ndim - 1, axis))
+    if axis < 0:
+        return axis + ndim
+    else:
+        return axis
