@@ -1,12 +1,10 @@
 import pytest
 pytest.importorskip('numpy')
 
-import dask.array as da
 from dask.optimize import fuse
 from dask.array.optimization import (getitem, optimize, optimize_slices,
-        fuse_slice)
+                                     fuse_slice)
 
-from dask.utils import raises
 from dask.array.core import getarray, getarray_nofancy
 
 
@@ -34,25 +32,24 @@ def test_fuse_getitem():
               (getarray_nofancy, 'x', (slice(1015, 1020), 10))),
 
              ((getarray, (getarray, 'x', (10, slice(1000, 2000))),
-                        (slice(15, 20),)),
+               (slice(15, 20), )),
               (getarray, 'x', (10, slice(1015, 1020)))),
 
              ((getarray, (getarray, 'x', (slice(1000, 2000), slice(100, 200))),
-                        (slice(None, None), slice(50, 60))),
+               (slice(None, None), slice(50, 60))),
               (getarray, 'x', (slice(1000, 2000), slice(150, 160)))),
 
              ((getarray, (getarray, 'x', (None, slice(None, None))),
-                        (slice(None, None), 5)),
+               (slice(None, None), 5)),
               (getarray, 'x', (None, 5))),
 
              ((getarray, (getarray, 'x', (slice(1000, 2000), slice(10, 20))),
-                        (slice(5, 10),)),
+               (slice(5, 10),)),
               (getarray, 'x', (slice(1005, 1010), slice(10, 20)))),
 
              ((getitem, (getitem, 'x', (slice(1000, 2000),)),
-                        (slice(5, 10), slice(10, 20))),
-              (getitem, 'x', (slice(1005, 1010), slice(10, 20))))
-        ]
+               (slice(5, 10), slice(10, 20))),
+              (getitem, 'x', (slice(1005, 1010), slice(10, 20))))]
 
     for inp, expected in pairs:
         result = optimize_slices({'y': inp})
@@ -92,18 +89,17 @@ def test_optimize_slicing():
 def test_fuse_slice():
     assert fuse_slice(slice(10, 15), slice(0, 5, 2)) == slice(10, 15, 2)
 
-    assert fuse_slice((slice(100, 200),), (None, slice(10, 20))) == \
-            (None, slice(110, 120))
-    assert fuse_slice((slice(100, 200),), (slice(10, 20), None)) == \
-            (slice(110, 120), None)
-    assert fuse_slice((1,), (None,)) == \
-            (1, None)
-    assert fuse_slice((1, slice(10, 20)), (None, None, 3, None)) == \
-            (1, None, None, 13, None)
+    assert (fuse_slice((slice(100, 200),), (None, slice(10, 20))) ==
+            (None, slice(110, 120)))
+    assert (fuse_slice((slice(100, 200),), (slice(10, 20), None)) ==
+            (slice(110, 120), None))
+    assert (fuse_slice((1,), (None,)) ==
+            (1, None))
+    assert (fuse_slice((1, slice(10, 20)), (None, None, 3, None)) ==
+            (1, None, None, 13, None))
 
-    assert (raises(NotImplementedError,
-                  lambda: fuse_slice(slice(10, 15, 2), -1)) or
-            fuse_slice(slice(10, 15, 2), -1) == 14)
+    with pytest.raises(NotImplementedError):
+        fuse_slice(slice(10, 15, 2), -1)
 
 
 def test_fuse_slice_with_lists():
@@ -116,17 +112,10 @@ def test_fuse_slice_with_lists():
 
 def test_hard_fuse_slice_cases():
     dsk = {'x': (getarray, (getarray, 'x', (None, slice(None, None))),
-                     (slice(None, None), 5))}
+                 (slice(None, None), 5))}
     assert optimize_slices(dsk) == {'x': (getarray, 'x', (None, 5))}
 
 
-def test_dont_fuse_different_slices():
-    x = da.random.random(size=(10, 10), chunks=(10, 1))
-    y = x.rechunk((1, 10))
-    dsk = optimize(y.dask, y._keys())
-    assert len(dsk) > 100
-
-
 def test_dont_fuse_fancy_indexing_in_getarray_nofancy():
     dsk = {'a': (getitem, (getarray_nofancy, 'x', (slice(10, 20, None), slice(100, 200, None))),
                  ([1, 3], slice(50, 60, None)))}
