@@ -544,7 +544,7 @@ def test_read_text_large_gzip():
         f.close()
 
         with pytest.raises(ValueError):
-            db.read_text(fn, blocksize=100, linedelimiter='\n')
+            db.read_text(fn, blocksize=50, linedelimiter='\n')
 
         c = db.read_text(fn)
         assert c.npartitions == 1
@@ -705,22 +705,25 @@ def test_to_dataframe():
     assert list(df4.compute()) == list(pd.DataFrame(list(b)))
 
 
-def test_to_textfiles():
-    b = db.from_sequence(['abc', '123', 'xyz'], npartitions=2)
-    for ext, myopen in [('gz', GzipFile), ('bz2', BZ2File), ('', open)]:
-        if ext == 'bz2' and PY2:
-            continue
-        with tmpdir() as dir:
-            c = b.to_textfiles(os.path.join(dir, '*.' + ext), compute=False)
-            dask.compute(*c)
-            assert os.path.exists(os.path.join(dir, '1.' + ext))
+ext_open = [('gz', GzipFile), ('', open)]
+if not PY2:
+    ext_open.append(('bz2', BZ2File))
 
-            f = myopen(os.path.join(dir, '1.' + ext), 'rb')
-            text = f.read()
-            if hasattr(text, 'decode'):
-                text = text.decode()
-            assert 'xyz' in text
-            f.close()
+
+@pytest.mark.parametrize('ext,myopen', ext_open)
+def test_to_textfiles(ext, myopen):
+    b = db.from_sequence(['abc', '123', 'xyz'], npartitions=2)
+    with tmpdir() as dir:
+        c = b.to_textfiles(os.path.join(dir, '*.' + ext), compute=False)
+        dask.compute(*c, get=dask.get)
+        assert os.path.exists(os.path.join(dir, '1.' + ext))
+
+        f = myopen(os.path.join(dir, '1.' + ext), 'rb')
+        text = f.read()
+        if hasattr(text, 'decode'):
+            text = text.decode()
+        assert 'xyz' in text
+        f.close()
 
 
 def test_to_textfiles_name_function_preserves_order():
@@ -1097,3 +1100,9 @@ def test_reductions_are_lazy():
     res = b.reduction(func, sum)
 
     assert res.compute(get=dask.get) == sum(range(10))
+
+
+def test_repeated_groupby():
+    b = db.range(10, npartitions=4)
+    c = b.groupby(lambda x: x % 3)
+    assert valmap(len, dict(c)) == valmap(len, dict(c))
