@@ -1,16 +1,20 @@
 from __future__ import print_function, division, absolute_import
 
 from .core import (DataFrame, Series, Index, _Frame, map_partitions,
-                   repartition)
-from .io import (from_array, from_bcolz, from_array, from_bcolz,
-                 from_pandas, from_dask_array, from_castra, read_hdf,
-                 from_imperative, from_delayed)
+                   repartition, to_delayed)
+from .io import (from_array, from_pandas, from_bcolz,
+                 from_dask_array, from_castra, read_hdf,
+                 from_delayed, read_csv, to_csv, read_table,
+                 demo, to_hdf, to_records, to_bag)
 from .optimize import optimize
-from .multi import merge, concat, melt
+from .multi import merge, concat
 from .rolling import (rolling_count, rolling_sum, rolling_mean, rolling_median,
                       rolling_min, rolling_max, rolling_std, rolling_var,
                       rolling_skew, rolling_kurt, rolling_quantile, rolling_apply,
                       rolling_window)
 from ..base import compute
-from .csv import read_csv
-from . import demo
+from .reshape import get_dummies, pivot_table, melt
+try:
+    from .io import read_parquet, to_parquet
+except ImportError:
+    pass
