@@ -15,6 +15,9 @@ from errno import ENOENT
 from collections import Iterator
 from contextlib import contextmanager
 from importlib import import_module
+from threading import Lock
+import uuid
+from weakref import WeakValueDictionary
 
 from .compatibility import (long, getargspec, BZ2File, GzipFile, LZMAFile, PY3,
                             urlsplit, unicode)
@@ -26,14 +29,6 @@ if system_encoding == 'ascii':
     system_encoding = 'utf-8'
 
 
-def raises(err, lamda):
-    try:
-        lamda()
-        return False
-    except err:
-        return True
-
-
 def deepmap(func, *seqs):
     """ Apply function inside nested lists
 
@@ -97,7 +92,8 @@ def tmpdir(dir=None):
     finally:
         if os.path.exists(dirname):
             if os.path.isdir(dirname):
-                shutil.rmtree(dirname)
+                with ignoring(OSError):
+                    shutil.rmtree(dirname)
             else:
                 with ignoring(OSError):
                     os.remove(dirname)
@@ -118,6 +114,28 @@ def filetext(text, extension='', open=open, mode='w'):
         yield filename
 
 
+@contextmanager
+def changed_cwd(new_cwd):
+    old_cwd = os.getcwd()
+    os.chdir(new_cwd)
+    try:
+        yield
+    finally:
+        os.chdir(old_cwd)
+
+
+@contextmanager
+def tmp_cwd(dir=None):
+    with tmpdir(dir) as dirname:
+        with changed_cwd(dirname):
+            yield dirname
+
+
+@contextmanager
+def noop_context():
+    yield
+
+
 def repr_long_list(seq):
     """
 
@@ -150,27 +168,33 @@ class IndexCallable(object):
 
 
 @contextmanager
-def filetexts(d, open=open, mode='t'):
+def filetexts(d, open=open, mode='t', use_tmpdir=True):
     """ Dumps a number of textfiles to disk
 
     d - dict
         a mapping from filename to text like {'a.csv': '1,1\n2,2'}
+
+    Since this is meant for use in tests, this context manager will
+    automatically switch to a temporary current directory, to avoid
+    race conditions when running tests in parallel.
     """
-    for filename, text in d.items():
-        f = open(filename, 'w' + mode)
-        try:
-            f.write(text)
-        finally:
+    with (tmp_cwd() if use_tmpdir else noop_context()):
+        for filename, text in d.items():
+            f = open(filename, 'w' + mode)
             try:
-                f.close()
-            except AttributeError:
-                pass
+                f.write(text)
+            finally:
+                try:
+                    f.close()
+                except AttributeError:
+                    pass
 
-    yield list(d)
+        yield list(d)
 
-    for filename in d:
-        if os.path.exists(filename):
-            os.remove(filename)
+        for filename in d:
+            if os.path.exists(filename):
+                with ignoring(OSError):
+                    os.remove(filename)
 
 
 compressions = {'gz': 'gzip', 'bz2': 'bz2', 'xz': 'xz'}
@@ -364,30 +388,25 @@ def pseudorandom(n, p, random_state=None):
     return out
 
 
-def different_seeds(n, random_state=None):
-    """ A list of different 32 bit integer seeds
+def random_state_data(n, random_state=None):
+    """Return a list of arrays that can initialize
+    ``np.random.RandomState``.
 
     Parameters
     ----------
-    n: int
-        Number of distinct seeds to return
-    random_state: int or np.random.RandomState
-        If int create a new RandomState with this as the seed
-    Otherwise draw from the passed RandomState
+    n : int
+        Number of tuples to return.
+    random_state : int or np.random.RandomState, optional
+        If an int, is used to seed a new ``RandomState``.
     """
     import numpy as np
 
     if not isinstance(random_state, np.random.RandomState):
         random_state = np.random.RandomState(random_state)
 
-    big_n = np.iinfo(np.int32).max
-
-    seeds = set(random_state.randint(big_n, size=n))
-    while len(seeds) < n:
-        seeds.add(random_state.randint(big_n))
-
-    # Sorting makes it easier to know what seeds are for what chunk
-    return sorted(seeds)
+    maxuint32 = np.iinfo(np.uint32).max
+    return [(random_state.rand(624) * maxuint32).astype('uint32')
+            for i in range(n)]
 
 
 def is_integer(i):
@@ -429,14 +448,16 @@ def file_size(fn, compression=None):
 
 
 ONE_ARITY_BUILTINS = set([abs, all, any, bool, bytearray, bytes, callable, chr,
-    classmethod, complex, dict, dir, enumerate, eval, float, format, frozenset,
-    hash, hex, id, int, iter, len, list, max, min, next, oct, open, ord, range,
-    repr, reversed, round, set, slice, sorted, staticmethod, str, sum, tuple,
-    type, vars, zip, memoryview])
+                          classmethod, complex, dict, dir, enumerate, eval,
+                          float, format, frozenset, hash, hex, id, int, iter,
+                          len, list, max, min, next, oct, open, ord, range,
+                          repr, reversed, round, set, slice, sorted,
+                          staticmethod, str, sum, tuple,
+                          type, vars, zip, memoryview])
 if PY3:
     ONE_ARITY_BUILTINS.add(ascii)  # noqa: F821
 MULTI_ARITY_BUILTINS = set([compile, delattr, divmod, filter, getattr, hasattr,
-    isinstance, issubclass, map, pow, setattr])
+                            isinstance, issubclass, map, pow, setattr])
 
 
 def takes_multiple_arguments(func):
@@ -491,25 +512,54 @@ class Dispatch(object):
     """Simple single dispatch."""
     def __init__(self):
         self._lookup = {}
+        self._lazy = {}
 
-    def register(self, type, func):
+    def register(self, type, func=None):
         """Register dispatch of `func` on arguments of type `type`"""
-        if isinstance(type, tuple):
-            for t in type:
-                self.register(t, func)
-        else:
-            self._lookup[type] = func
+        def wrapper(func):
+            if isinstance(type, tuple):
+                for t in type:
+                    self.register(t, func)
+            else:
+                self._lookup[type] = func
+            return func
+
+        return wrapper(func) if func is not None else wrapper
+
+    def register_lazy(self, toplevel, func=None):
+        """
+        Register a registration function which will be called if the
+        *toplevel* module (e.g. 'pandas') is ever loaded.
+        """
+        def wrapper(func):
+            self._lazy[toplevel] = func
+            return func
+
+        return wrapper(func) if func is not None else wrapper
 
     def __call__(self, arg):
-        # We dispatch first on type(arg), and fall back to iterating through
-        # the mro. This is significantly faster in the common case where
-        # type(arg) is in the lookup, with only a small penalty on fall back.
+        # Fast path with direct lookup on type
         lk = self._lookup
         typ = type(arg)
-        if typ in lk:
-            return lk[typ](arg)
+        try:
+            impl = lk[typ]
+        except KeyError:
+            pass
+        else:
+            return impl(arg)
+        # Is a lazy registration function present?
+        toplevel, _, _ = typ.__module__.partition('.')
+        try:
+            register = self._lazy.pop(toplevel)
+        except KeyError:
+            pass
+        else:
+            register()
+            return self(arg)  # recurse
+        # Walk the MRO and cache the lookup result
         for cls in inspect.getmro(typ)[1:]:
             if cls in lk:
+                lk[typ] = lk[cls]
                 return lk[cls](arg)
         raise TypeError("No dispatch for {0} type".format(typ))
 
@@ -526,12 +576,22 @@ def ensure_not_exists(filename):
 
 
 def _skip_doctest(line):
-    if '>>>' in line:
+    # NumPy docstring contains cursor and comment only example
+    stripped = line.strip()
+    if stripped == '>>>' or stripped.startswith('>>> #'):
+        return stripped
+    elif '>>>' in stripped:
         return line + '    # doctest: +SKIP'
     else:
         return line
 
 
+def skip_doctest(doc):
+    if doc is None:
+        return ''
+    return '\n'.join([_skip_doctest(line) for line in doc.split('\n')])
+
+
 def derived_from(original_klass, version=None, ua_args=[]):
     """Decorator to attach original class's docstring to the wrapped method.
 
@@ -556,10 +616,13 @@ def derived_from(original_klass, version=None, ua_args=[]):
             if doc is None:
                 doc = ''
 
-            method_args = getargspec(method).args
-            original_args = getargspec(original_method).args
+            try:
+                method_args = getargspec(method).args
+                original_args = getargspec(original_method).args
+                not_supported = [m for m in original_args if m not in method_args]
+            except TypeError:
+                not_supported = []
 
-            not_supported = [m for m in original_args if m not in method_args]
             if len(ua_args) > 0:
                 not_supported.extend(ua_args)
 
@@ -568,7 +631,7 @@ def derived_from(original_klass, version=None, ua_args=[]):
                         "        Dask doesn't supports following argument(s).\n\n")
                 args = ''.join(['        * {0}\n'.format(a) for a in not_supported])
                 doc = doc + note + args
-            doc = '\n'.join([_skip_doctest(line) for line in doc.split('\n')])
+            doc = skip_doctest(doc)
             method.__doc__ = doc
             return method
 
@@ -586,17 +649,33 @@ def derived_from(original_klass, version=None, ua_args=[]):
     return wrapper
 
 
-def funcname(func, full=False):
+def funcname(func):
     """Get the name of a function."""
-    while hasattr(func, 'func'):
-        func = func.func
+    # functools.partial
+    if isinstance(func, functools.partial):
+        return funcname(func.func)
+    # methodcaller
+    if isinstance(func, methodcaller):
+        return func.method
+
+    module_name = getattr(func, '__module__', None) or ''
+    type_name = getattr(type(func), '__name__', None) or ''
+
+    # toolz.curry
+    if 'toolz' in module_name and 'curry' == type_name:
+        return func.func_name
+    # multipledispatch objects
+    if 'multipledispatch' in module_name and 'Dispatcher' == type_name:
+        return func.name
+
+    # All other callables
     try:
-        if full:
-            return func.__qualname__.strip('<>')
-        else:
-            return func.__name__.strip('<>')
+        name = func.__name__
+        if name == '<lambda>':
+            return 'lambda'
+        return name
     except:
-        return str(func).strip('<>')
+        return str(func)
 
 
 def ensure_bytes(s):
@@ -613,8 +692,26 @@ def ensure_bytes(s):
         return s
     if hasattr(s, 'encode'):
         return s.encode()
-    raise TypeError(
-            "Object %s is neither a bytes object nor has an encode method" % s)
+    msg = "Object %s is neither a bytes object nor has an encode method"
+    raise TypeError(msg % s)
+
+
+def ensure_unicode(s):
+    """ Turn string or bytes to bytes
+
+    >>> ensure_unicode(u'123')
+    u'123'
+    >>> ensure_unicode('123')
+    u'123'
+    >>> ensure_unicode(b'123')
+    u'123'
+    """
+    if isinstance(s, unicode):
+        return s
+    if hasattr(s, 'decode'):
+        return s.decode()
+    msg = "Object %s is neither a bytes object nor has an encode method"
+    raise TypeError(msg % s)
 
 
 def digit(n, k, base):
@@ -850,6 +947,11 @@ class methodcaller(object):
     def __reduce__(self):
         return (methodcaller, (self.method,))
 
+    def __str__(self):
+        return "<%s: %s>" % (self.__class__.__name__, self.method)
+
+    __repr__ = __str__
+
 
 class MethodCache(object):
     """Attribute access on this object returns a methodcaller for that
@@ -866,3 +968,67 @@ class MethodCache(object):
 
 
 M = MethodCache()
+
+
+class SerializableLock(object):
+    _locks = WeakValueDictionary()
+    """ A Serializable per-process Lock
+
+    This wraps a normal ``threading.Lock`` object and satisfies the same
+    interface.  However, this lock can also be serialized and sent to different
+    processes.  It will not block concurrent operations between processes (for
+    this you should look at ``multiprocessing.Lock`` or ``locket.lock_file``
+    but will consistently deserialize into the same lock.
+
+    So if we make a lock in one process::
+
+        lock = SerializableLock()
+
+    And then send it over to another process multiple times::
+
+        bytes = pickle.dumps(lock)
+        a = pickle.loads(bytes)
+        b = pickle.loads(bytes)
+
+    Then the deserialized objects will operate as though they were the same
+    lock, and collide as appropriate.
+
+    This is useful for consistently protecting resources on a per-process
+    level.
+
+    The creation of locks is itself not threadsafe.
+    """
+    def __init__(self, token=None):
+        self.token = token or str(uuid.uuid4())
+        if self.token in SerializableLock._locks:
+            self.lock = SerializableLock._locks[self.token]
+        else:
+            self.lock = Lock()
+            SerializableLock._locks[self.token] = self.lock
+
+    def acquire(self, *args):
+        return self.lock.acquire(*args)
+
+    def release(self, *args):
+        return self.lock.release(*args)
+
+    def __enter__(self):
+        self.lock.__enter__()
+
+    def __exit__(self, *args):
+        self.lock.__exit__(*args)
+
+    @property
+    def locked(self):
+        return self.locked
+
+    def __getstate__(self):
+        return self.token
+
+    def __setstate__(self, token):
+        self.__init__(token)
+
+    def __str__(self):
+        return "<%s: %s>" % (self.__class__.__name__, self.token)
+
+    __repr__ = __str__
